%Tristan Ursell
%All-to-All survival plots
%May 2019 -- use this version

clearvars
close all
clc
drawnow

%get *.mat files in current folder
[file1, path1, ~] = uigetfile({'*.mat'}, 'Pick a file');

%get all files
dir1=dir(fullfile(path1,'*.mat'));

load(fullfile(path1,[file1]))

if ~exist('info1','var')
    %set up info matrix
    %info1(S,Lbox,Pmean,dP,R,dx,xrms,rep,end_time,meanA(end),meanB(end),...,meanL(end));
    info1=zeros(length(dir1),9+12);
    
    for i=1:length(dir1)
        clearvars -except i file1 path1 dir1 info1 Lbox dx Pmean
        fullname1=fullfile(path1,dir1(i).name);
        
        if dir1(i).bytes<10e3
            disp([dir1(i).name ' appears to be empty, and will be deleted.'])
            dir1(i).valid = 0;
            
            delete(fullname1)
        else
            try
                load(fullname1)
            catch
                disp([dir1(i).name ' appears to be corrupt, and will be deleted.'])
                dir1(i).valid = 0;
                
                delete(fullname1)
                continue
            end
            
            %set as valid file
            dir1(i).valid = 1;
            
            info1(i,1) = S;      %number of species
            info1(i,2) = Lbox;   %box size (pixels)
            info1(i,3) = Pmean;  %mean competition parameter
            info1(i,4) = dP;     %width of competition variation
            info1(i,5) = R;      %radius of pillars
            info1(i,6) = dx;     %center to center spacing of pillars
            info1(i,7) = xrms;   %grid asymmetry parameter
            info1(i,8) = rep;    %random number seed
            
            %find last time step recorded
            temp1=meanA+meanB+meanC+meanD+meanE+meanF+meanG+meanH;
            t_temp = find(temp1==0,1,'first') - 1;
            if isempty(t_temp)
                t_temp = length(meanA);
            end
            info1(i,9) = t_temp; %last simulated time point
            
            %end point concentrations
            info1(i,10) = meanA(t_temp);
            info1(i,11) = meanB(t_temp);
            info1(i,12) = meanC(t_temp);
            info1(i,13) = meanD(t_temp);
            info1(i,14) = meanE(t_temp);
            info1(i,15) = meanF(t_temp);
            info1(i,16) = meanG(t_temp);
            info1(i,17) = meanH(t_temp);
            info1(i,18) = meanI(t_temp);
            info1(i,19) = meanJ(t_temp);
            info1(i,20) = meanK(t_temp);
            info1(i,21) = meanL(t_temp);
        end
        
        disp([num2str(i) ' / ' num2str(length(dir1))])
    end
    valid_vec=[dir1(:).valid];
end

%%{
%check for duplicates
dup_check=zeros(length(dir1),length(dir1));
for i=1:length(dir1)
    if dir1(i).valid
        for j=i:length(dir1)
            if dir1(j).valid
                cond_vec = abs(info1(i,1:8)-info1(j,1:8))<1e-6;
                if all(cond_vec)
                    dup_check(i,j)=1;
                end
            end
        end
    end
    disp(['Checking duplicates ' num2str(i) ' of ' num2str(length(dir1)) '.'])
end
if sum(dup_check(:))>sum(valid_vec)
    warning('Duplicate conditions detected.')
end
%}

%organize data by total number of surviving species under current conditions
%***  USING params_generator.m FOR THIS DATA SET
%***

%count parameter variations
disp(['Found ' num2str(sum(valid_vec)) ' valid mat-files out of ' num2str(length(dir1)) ' total mat-files.'])
S_num     = length(unique(info1(valid_vec==1,1)));
Lbox_num  = length(unique(info1(valid_vec==1,2)));
Pmean_num = length(unique(info1(valid_vec==1,3)));
dP_num    = length(unique(info1(valid_vec==1,4)));
R_num     = length(unique(info1(valid_vec==1,5)));
dx_num    = length(unique(info1(valid_vec==1,6)));
xrms_num  = length(unique(info1(valid_vec==1,7)));
rep_num   = length(unique(info1(valid_vec==1,8)));

%check for proper number of parameter variations
%if S_num~=1
%    warning('The parameter ''S'' seems to vary when it should not.')
%end
%if Lbox_num~=1
%    warning('The parameter ''Lbox'' seems to vary when it should not.')
%end
if Pmean_num~=1
    warning('The parameter ''Pmean'' seems to vary when it should not.')
end
if dP_num~=1
    warning('The parameter ''dP'' seems to vary when it should not.')
end
if R_num~=1
    warning('The parameter ''R'' seems to vary when it should not.')
end
if dx_num~=1
    warning('The parameter ''dx'' seems to vary when it should not.')
end
%if xrms_num~=1
%    warning('The parameter ''xrms'' seems to vary when it should not.')
%end
if rep_num~=sum(valid_vec)
    warning('The ''rep'' parameter may have repeats.')
end

%parameter space
dS_vec = [2 : 1: 12]; %#ok
dL_vec = [500,1000];
dxrms_vec = [0:0.5:1]; %#ok

%generate data matrix
species_avg = zeros(length(dS_vec),length(dL_vec),length(dxrms_vec)); %number of species surviving at terminus of each simulation
species_med = zeros(size(species_avg));
species_min = zeros(size(species_avg));
species_max = zeros(size(species_avg));
num_reps = zeros(size(species_avg));

species_surv = zeros(length(dS_vec),length(dL_vec),length(dxrms_vec),max(dS_vec));

%coalesce data
for i=1:length(dS_vec)
    for j=1:length(dL_vec)
        for m=1:length(dxrms_vec)
            
            %estimate minimum abundance (for mean cutoff)
            abd_min = 1/8*dx^2/dL_vec(j)^2;
            
            %find indices of current parameter sets
            cond1=abs(info1(:,1)-dS_vec(i))<1e-6; %<----
            cond2=abs(info1(:,2)-dL_vec(j))<1e-6; %<----
            cond3=abs(info1(:,7)-dxrms_vec(m))<1e-6; %<----
            list1=find(and(and(cond1,cond2),cond3)); %<----
            
            if ~isempty(list1)
                num_reps(i,j,m) = length(list1); %number of sims for these parameters
                
                curr_num=zeros(1,length(list1));
                for k=1:length(list1)
                    %get species abundances
                    abd_temp = info1(list1(k),10:21);
                    
                    curr_num(k) = sum(abd_temp >= abd_min); % number of species surviving at sim's end
                end
                
                %{
                %choose random 50
                rep_size = 50;
                temp1 = rand(1,length(curr_num));
                [~, rand1] = sort(temp1);
                
                if num_reps(i,j,m)<rep_size
                    rep_size = num_reps(i,j,m);
                end
                
                % mean number of species surviving at sim's end
                species_avg(i,j,m) = mean(curr_num(rand1(1:rep_size )));
                species_med(i,j,m) = median(curr_num(rand1(1:rep_size )));
                species_min(i,j,m) = min(curr_num(rand1(1:rep_size )));
                species_max(i,j,m) = max(curr_num(rand1(1:rep_size )));
                %}
                
                %get number of surviving species (histogram) data
                species_surv(i,j,m,:)=hist(curr_num,1:max(dS_vec));
                
                % mean number of species surviving at sim's end
                species_avg(i,j,m) = mean(curr_num);
                species_med(i,j,m) = median(curr_num);
                species_min(i,j,m) = min(curr_num);
                species_max(i,j,m) = max(curr_num);
            else
                disp('hmmm ...')
            end
        end
    end
    disp([num2str(i) ' \ ' num2str(length(dS_vec))])
end

%{
%average number of surviving species
h2=figure;
set(h2,'Position',[500 500 650 500])
cmap1 = hsv(length(dP_vec)+1);
for i=1:length(dP_vec)
    hold on
    plot(dS_vec,species_avg(i,:)./dS_vec,'-r.','linewidth',3,'color',cmap1(i,:),'markersize',30)
    %errorbar
    label_vec(i) = {['\Delta P / <P> = ' num2str(dP_vec(i)/Pmean)]};
end
xlabel('number of species')
ylabel('mean no. of surviving species / initial no. of species')
ylim([0.8 1.05])
xlim([1.5 12.5])
xticks(dS_vec)
title(['n = ' num2str(50)])
legend(label_vec,'Location','southwest','Orientation','vertical')
box on
%}

h3 =figure;
hold on
plot(2:12,species_avg(:,1,1)'./(2:12),'b','linewidth',2)
plot(2:12,species_avg(:,2,1)'./(2:12),'r','linewidth',2)
plot(2:12,species_avg(:,1,2)'./(2:12),'b-.','linewidth',2)
plot(2:12,species_avg(:,2,2)'./(2:12),'r-.','linewidth',2)
plot(2:12,species_avg(:,1,end)'./(2:12),'b--','linewidth',2)
plot(2:12,species_avg(:,2,end)'./(2:12),'r--','linewidth',2)
label_vec(1) = {['L = ' num2str(500) ', \delta = ' num2str(0)]};
label_vec(2) = {['L = ' num2str(1000) ', \delta = ' num2str(0)]};
label_vec(3) = {['L = ' num2str(500) ', \delta = ' num2str(0.5)]};
label_vec(4) = {['L = ' num2str(1000) ', \delta = ' num2str(0.5)]};
label_vec(5) = {['L = ' num2str(500) ', \delta = ' num2str(1)]};
label_vec(6) = {['L = ' num2str(1000) ', \delta = ' num2str(1)]};
legend(label_vec,'Location','southwest','Orientation','vertical')
xlabel('initial number of species')
ylabel('mean no. of surviving species / initial no. of species')
box on
ylim([0.65 1.05])
xlim([1.5 12.5])
title(['\Delta P/<P> = ' num2str(dP/Pmean)])


%*************************************************************************
%*************    PARAMETER ESTIMATION   *********************************
%*************************************************************************

%parameter vectors
alpha_vec = zeros(length(dS_vec),length(dL_vec),length(dxrms_vec));
alpha_plus = zeros(size(alpha_vec));
alpha_minus = zeros(size(alpha_vec));

%confidence interval
CI = 0.95;

%input vectors
X = 1:max(dS_vec);

%parametric test vector
alpha_temp = 0:0.0001:1;

%loop through all conditions
for i=1:length(dS_vec)
    for j=1:length(dL_vec)
        for m=1:length(dxrms_vec)
            
            %total number of simulations in current parameter set
            M = num_reps(i,j,m);
            
            %survival distribution
            Y = squeeze(species_surv(i,j,m,:));
            
            %normalize Y (assuming dX = 1)
            Y=Y/sum(Y);
            
            %calculate total number of survivals
            V = sum(X(:).*Y(:))*M;
            
            %likelihood (unormalized)
            logMLE = V*log10(alpha_temp) + (M*dS_vec(i) - V)*log10(1-alpha_temp);
            temp0 = logMLE - max(logMLE);
            MLE0 = 10.^temp0;
            
            %cumulative normalized
            MLE_cum = cumsum(MLE0)/max(cumsum(MLE0));
            
            %maximum
            temp1 = find(MLE0 == max(MLE0),1,'last');
            alpha_vec(i,j,m) = alpha_temp(temp1);
            
            %lower bound
            temp1 = find((MLE_cum-(1-CI)/2).^2 == min((MLE_cum-(1-CI)/2).^2),1,'first');
            alpha_minus(i,j,m) = alpha_vec(i,j,m) - alpha_temp(temp1);
            
            %upper bound
            temp1 = find((MLE_cum-(CI + (1-CI)/2)).^2 == min((MLE_cum-(CI + (1-CI)/2)).^2),1,'first');
            alpha_plus(i,j,m) = alpha_temp(temp1) - alpha_vec(i,j,m);
        end
    end
end

%*************************************************************************
%plotting MLE fit
%average number of surviving species
h3 =figure;
hold on

%L=100, delta=0
j=1; m=1;
p = alpha_vec(:,j,m);
p_plus = p + alpha_plus(:,j,m);
p_minus = p - alpha_minus(:,j,m);
plot(2:12,p,'b-','linewidth',1)
plot(2:12,p_plus,'b--','linewidth',1)
plot(2:12,p_minus,'b--','linewidth',1)
label_vec(1) = {['L = ' num2str(dL_vec(j)) ', \delta = ' num2str(dxrms_vec(m))]};

%L=100, delta=0.5
j=1; m=2;
p = alpha_vec(:,j,m);
p_plus = p + alpha_plus(:,j,m);
p_minus = p - alpha_minus(:,j,m);
plot(2:12,p,'b-','linewidth',2)
plot(2:12,p_plus,'b--','linewidth',2)
plot(2:12,p_minus,'b--','linewidth',2)
label_vec(2) = {['L = ' num2str(dL_vec(j)) ', \delta = ' num2str(dxrms_vec(m))]};

%L=100, delta=1
j=1; m=3;
p = alpha_vec(:,j,m);
p_plus = p + alpha_plus(:,j,m);
p_minus = p - alpha_minus(:,j,m);
plot(2:12,p,'b-','linewidth',3)
plot(2:12,p_plus,'b--','linewidth',3)
plot(2:12,p_minus,'b--','linewidth',3)
label_vec(3) = {['L = ' num2str(dL_vec(j)) ', \delta = ' num2str(dxrms_vec(m))]};

%L=200, delta=0
j=2; m=1;
p = alpha_vec(:,j,m);
p_plus = p + alpha_plus(:,j,m);
p_minus = p - alpha_minus(:,j,m);
plot(2:12,p,'r-','linewidth',1)
plot(2:12,p_plus,'r--','linewidth',1)
plot(2:12,p_minus,'r--','linewidth',1)
label_vec(4) = {['L = ' num2str(dL_vec(j)) ', \delta = ' num2str(dxrms_vec(m))]};

%L=200, delta=0.5
j=2; m=2;
p = alpha_vec(:,j,m);
p_plus = p + alpha_plus(:,j,m);
p_minus = p - alpha_minus(:,j,m);
plot(2:12,p,'r-','linewidth',2)
plot(2:12,p_plus,'r--','linewidth',2)
plot(2:12,p_minus,'r--','linewidth',2)
label_vec(5) = {['L = ' num2str(dL_vec(j)) ', \delta = ' num2str(dxrms_vec(m))]};

%L=200, delta=1
j=2; m=3;
p = alpha_vec(:,j,m);
p_plus = p + alpha_plus(:,j,m);
p_minus = p - alpha_minus(:,j,m);
plot(2:12,p,'r-','linewidth',3)
plot(2:12,p_plus,'r--','linewidth',3)
plot(2:12,p_minus,'r--','linewidth',3)
label_vec(6) = {['L = ' num2str(dL_vec(j)) ', \delta = ' num2str(dxrms_vec(m))]};

legend(label_vec,'Location','southwest','Orientation','vertical')
xlabel('initial number of species')
ylabel('mean no. of surviving species / initial no. of species')
box on
ylim([0.65 1.05])
xlim([1.5 12.5])
title(['\Delta P/<P> = ' num2str(dP/Pmean)])






%*************************************************************************
%***** CREATE FIXER PARAMETER FILE ***************************************
%*************************************************************************
return

%FIGURE
filename = 'params_figure_4B_fix.txt';
fileID = fopen(filename,'w');

% Input parameters:

% S = number of species to initialize with (at least 2, up to 6)
S = [2 : 1 : 12]; %#ok

% N = number of time steps in simulation (N*dt = # doubling times)
N = 500000;

% Lbox = size of the simulation box (pixels)
Lbox = [500, 1000];

% DC = diffusion coefficient (dimensionless)
DC = 15;

% Pmean = mean interaction strength (dimensionless)
Pmean = 0.25;

% dP = width of interaction parameter Pmean uniform distribution
dP = 0.1;

% dsc = discrete cutoff below which fields are set to 0
dsc =0;

% pillarq = logical (1,0); should pillars be included?
pillarq = 1;

% R = pillar radius (pixels)
R = 15;

% dx_mult = center-to-center pillar spacing (units of R)
dx_mult = 3.5;

% xrms = grid jitter on a uniform distribution (0.. ~1) as a fraction (dx-R)/2
xrms = [0:0.5:1];

% rep = seed used for random number generator (integer)
rep = 0;

%number of independnet replicates
Nrep = 50;

%total number of simulations
Ntot = length(S)*length(N)*length(Lbox)*length(DC)*length(Pmean)*length(dP)*length(dsc)*length(R)*length(dx_mult)*length(xrms)*length(rep)*Nrep;

%construct parameter file
q=0;
for j=1:length(S)
    for k=1:length(Lbox)
        for m=1:length(xrms)
            %get current number of reps at this grid point
            for i=1:(Nrep-num_reps(j,k,m))
                q=q+1;
                
                txt1=[num2str(S(j)) ',' num2str(N) ',' num2str(Lbox(k)) ',' num2str(DC) ','...
                    num2str(Pmean) ',' num2str(dP) ',' num2str(dsc) ',' num2str(pillarq) ',' ...
                    num2str(R) ',' num2str(dx_mult) ',' num2str(xrms(m)) ',' num2str(round(rand*1e9))];
                disp([num2str(q) ' / ' num2str(Ntot) ' : ' txt1])
                drawnow
                
                fprintf(fileID,[txt1 '\n']);
            end
        end
    end
end
fclose all;

movefile(filename,[filename(1:end-4) '_' num2str(q) '.txt'])

disp(['Done writing: ' filename ', Total sims: ' num2str(q)])










