%Tristan Ursell
%Create Parameters File for HPC All-to-all paper sims
%Dec 2018
%

%FIGURE
filename = 'params_figure_4B.txt';
fileID = fopen(filename,'w');

% Input parameters:

% S = number of species to initialize with (at least 2, up to 6)
S = [2 : 1: 12]; %#ok

% N = number of time steps in simulation (N*dt = # doubling times)
N = 500000;

% Lbox = size of the simulation box (pixels)
Lbox = [500,1000];

% DC = diffusion coefficient (dimensionless)
DC = 15;

% Pmean = mean interaction strength (dimensionless)
Pmean = 0.25;

% dP = width of interaction parameter Pmean uniform distribution
dP = 0.1;

% dsc = discrete cutoff below which fields are set to 0
dsc =0;

% pillarq = logical (1,0); should pillars be included?
pillarq = 1;

% R = pillar radius (pixels)
R = 15;

% dx_mult = center-to-center pillar spacing (units of R)
dx_mult = 3.5;

% xrms = grid jitter on a uniform distribution (0.. ~1) as a fraction (dx-R)/2
xrms = [0:0.5:1];

% rep = seed used for random number generator (integer)
rep = 0;

%%%%%%
%number of independnet replicates
Nrep = 50;

%pick up at sim = q_pickup
q_pickup = 2500;

%total number of simulations
Ntot = length(S)*length(N)*length(Lbox)*length(DC)*length(Pmean)*length(dP)*length(dsc)*length(pillarq)*length(R)*length(dx_mult)*length(xrms)*length(rep)*Nrep;

%construct parameter file
q=0; q_sim=0;
for q1=S
    for q2=Lbox
        for q3=dP
            for q4=R
                for q5=dx_mult
                    for q6=xrms
                        for q7=1:Nrep
                            
                            q=q+1;
                            if q>=q_pickup
                                
                                q_sim = q_sim + 1;
                                
                                txt1=[num2str(q1) ',' num2str(N) ',' num2str(q2) ',' num2str(DC) ','...
                                    num2str(Pmean) ',' num2str(q3) ',' num2str(dsc) ',' num2str(pillarq) ',' ...
                                    num2str(q4) ',' num2str(q5) ',' num2str(q6) ',' num2str(rep)];
                                disp(txt1)
                                drawnow
                                
                                fprintf(fileID,[txt1 '\n']);
                            end
                        end
                    end
                end
            end
        end
    end
end
fclose all;
disp(['Done writing: ' filename ', Total sims: ' num2str(q_sim)])















